<?php
namespace Neuron\Person\Controller\Api;

use Neuron\Actor;
use Neuron\Composite;
use Neuron\Generic\Result;  
use Neuron\Application\Framework\Entity;
use Neuron\Person\Skill as mSkill;
use Neuron\Person\Skills;
use Neuron\Person\Skill\Type;
use Neuron\Person\Skill\Types;
use Neuron\Person\Parameters;
use Neuron\Person\Parameter;

class Skill extends \Neuron\Person\Controller {
    const EVENT_SUBJECT = 'Person';
    const SUCCESS_CODE = 0;
    const FAIL_CODE = 1;
    const FAIL_CODE_SESSION = 11;

    const CODE_FIELD_REQUIRED  = 400;
    const CODE_UNAUTHORIZED = 401;
    const CODE_SESSION_EXPIRED = 403;

    public function listAction() {
		$mSkill     = new Skills(mSKill\Storage::factory($this->getDb(), $this->getConfig()));
        $result     = new Result();
        
        $request    = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
        }

        $data = $this->getREST()->getParameters();
        $guid = $this->getGuid();
		
        try {
            if($guid == "" || empty($guid)){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $entity = Entity::get($this, $guid);
            $user = $entity->user();
            if(!$user->id){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result); 
            }

            $userPerson          = $this->getUserPerson($user);

            $params['start']    = 0;
            $params['length']   = 10000000;
            $params['draw']     = 1;
            $params['records']  = 10000000;
            $params['person_id']= $userPerson['person']['ref_id'];

            $keyword            = isset($data['search']['value']) ? $data['search']['value'] : null;
            
            $arr                = $mSkill->load($keyword, \Neuron\Person\Skill\Storage::LISTBY_PAGE, $params);
            $array              = [];
            
            if (isset($arr->data['list'])) {
                foreach ($arr->data['list'] as $rows) {
                    if ($rows['skill_value'] == 1) {
                        $val = 'Master'; $col = '#9400D3';
                    } elseif ($rows['skill_value'] == 2) {
                        $val = 'Expert'; $col = '#C0C0C0';
                    } elseif ($rows['skill_value'] == 3) {
                        $val = 'Intermediate'; $col = '#FFD700';
                    } elseif ($rows['skill_value'] == 4) {
                        $val = 'Based'; $col = '#BE6D03';
                    }

                    $rows['params_name']    = $val;
                    $rows['params_color']   = $col;

                    $array[]                = $rows;
                }
            }
            
			$requestPost = array(
				'guid' => $guid,
				'code' => '',
				'info' => '',
				'data' => $data
			); 
			
            if(!empty($data['draw'])){
                echo json_encode($arr);
				
				$logparam['subject'] = 'API PERSON LISTSKILL';
				$logparam['identity'] = $user->id ? $user->id : '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api person list skill';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($arr);
				
				$this->writeLogAction($logparam);
				
                return $this->getResponse();
            }else{
                $result->guid = $guid;
                $result->code = self::SUCCESS_CODE;
                $result->data = $array;
				
				$responseResult = array(
					'guid' => $result->guid,
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API PERSON LISTSKILL';
				$logparam['identity'] = $user->id ? $user->id : '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api person list skill';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);
				
				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);
            }
        } catch (Exception $ex) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
            return $this->getREST()->getResponse($result);
        }
    }

    public function listTypeAction() {
		$mSkill     = new Types(Type\Storage::factory($this->getDb(), $this->getConfig()));
        $result     = new Result();
        
        $request    = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
        }

        $data = $this->getREST()->getParameters();
        $guid = $this->getGuid();
		
        try {
            if($guid == "" || empty($guid)){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $entity = Entity::get($this, $guid);
            $user = $entity->user();
            if(!$user->id){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result); 
            }

            $userPerson          = $this->getUserPerson($user);

            $params['start']    = 0;
            $params['length']   = 10000000;
            $params['draw']     = 1;
            $params['records']  = 10000000;

            $keyword            = isset($data['search']['value']) ? $data['search']['value'] : null;
            
            $arr                = $mSkill->load($keyword, \Neuron\Person\Skill\Type\Storage::LISTBY_PAGE, $params);
            
			$requestPost = array(
				'guid' => $guid,
				'code' => '',
				'info' => '',
				'data' => $data
			); 
			
            if(!empty($data['draw'])){
                echo json_encode($arr);
				
				$logparam['subject'] = 'API PERSON LISTSKILLTYPE';
				$logparam['identity'] = $user->id ? $user->id : '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api person list skill type';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($arr);
				
				$this->writeLogAction($logparam);
				
                return $this->getResponse();
            }else{
                $result->guid = $guid;
                $result->code = self::SUCCESS_CODE;
                $result->data = $arr->data['list'];
				
				$responseResult = array(
					'guid' => $result->guid,
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API PERSON LISTSKILLTYPE';
				$logparam['identity'] = $user->id ? $user->id : '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api person list skill type';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);
				
				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);
            }
        } catch (Exception $ex) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
            return $this->getREST()->getResponse($result);
        }
    }

    public function listParamsAction() {
		$mSkill     = new Parameters(Parameter\Storage::factory($this->getDb(), $this->getConfig()));
        $result     = new Result();
        
        $request    = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
        }

        $data = $this->getREST()->getParameters();
        $guid = $this->getGuid();
		
        try {
            if($guid == "" || empty($guid)){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $entity = Entity::get($this, $guid);
            $user = $entity->user();
            if(!$user->id){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result); 
            }

            $userPerson         = $this->getUserPerson($user);

            $keyword                    = 'SKILL_LEVEL';
            $params['search']['value']  = isset($data['search']['value']) ? $data['search']['value'] : null;

            $arr                = $mSkill->listparam($keyword, $params);
            $params             = array();
            if ($arr) {
                foreach ($arr as $data) {
                    $row['param_ref_id']    = $data['param_ref_id'];
                    $row['value1']          = $data['value1'];
                    array_push($params, $row);
                }
            }
            
			$requestPost = array(
				'guid' => $guid,
				'code' => '',
				'info' => '',
				'data' => $data
			); 
			
            if(!empty($data['draw'])){
                echo json_encode($params);
				
				$logparam['subject'] = 'API PERSON LISTSKILLTYPE';
				$logparam['identity'] = $user->id ? $user->id : '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api person list skill type';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($params);
				
				$this->writeLogAction($logparam);
				
                return $this->getResponse();
            }else{
                $result->guid = $guid;
                $result->code = self::SUCCESS_CODE;
                $result->data = $params;
				
				$responseResult = array(
					'guid' => $result->guid,
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API PERSON LISTSKILLTYPE';
				$logparam['identity'] = $user->id ? $user->id : '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api person list skill type';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);
				
				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);
            }
        } catch (Exception $ex) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
            return $this->getREST()->getResponse($result);
        }
    }

    public function saveAction() {
		$mSkill     = new mSKill(mSKill\Storage::factory($this->getDb(), $this->getConfig()));
        $result     = new Result();
        
        $request    = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
        }

        $data = $this->getREST()->getParameters();
        $guid = $this->getGuid();
		
        try {
            if($guid == "" || empty($guid)){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $entity = Entity::get($this, $guid);
            $user = $entity->user();
            if(!$user->id){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result); 
            }

            $userPerson         = $this->getUserPerson($user);
            $sessmgt 		    = $this->getSession()->owner();

            $data               = $this->getPostParams();

            $data['start_date']	= date('Y-m-d');
            $data['update_dtm']	= date('Y-m-d H:m:s');
            $data['update_by']	= $sessmgt;
            $data['person_id']	= $userPerson['person']['ref_id'];
            
            $mSkill->load(null, mSkill\Storage::LOADBY_PARAMS, $data);

            if (empty($data['person_skill_id'])) {
                if ($mSkill->person_skill_id) {
                    $result->code = 401;
                    $result->info = 'skill already exists';
                } else {
                    $result       = $mSkill->create($data);
                }
            } else {
                $result       = $mSkill->create($data);
            }
            
			$requestPost = array(
				'guid' => $guid,
				'code' => '',
				'info' => '',
				'data' => $data
			); 
			
            $result->guid = $guid;
            $result->info = $this->translate($result->info);
            
            $responseResult = array(
                'guid' => $result->guid,
                'code' => $result->code,
                'info' => $result->info,
                'data' => $result->data
            ); 

            $logparam['subject'] = 'API PERSON SAVESKILL';
            $logparam['identity'] = $user->id ? $user->id : '';
            $logparam['user'] = $user->id ? $user->id : '';
            $logparam['description'] = 'Log proses api person save skill';
            $logparam['request'] = json_encode($requestPost);
            $logparam['response'] = json_encode($responseResult);
            
            $this->writeLogAction($logparam);
            
            return $this->getREST()->getResponse($result);
        } catch (Exception $ex) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
            return $this->getREST()->getResponse($result);
        }
    }

    public function deleteAction() {
		$mSkill     = new mSKill(mSKill\Storage::factory($this->getDb(), $this->getConfig()));
        $result     = new Result();
        
        $request    = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
        }

        $data = $this->getREST()->getParameters();
        $guid = $this->getGuid();
		
        try {
            if($guid == "" || empty($guid)){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $entity = Entity::get($this, $guid);
            $user = $entity->user();
            if(!$user->id){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result); 
            }

            $userPerson = $this->getUserPerson($user);
            $data       = $this->getPostParams();

            $result     = $mSkill->delete($data['person_skill_id']);
            
			$requestPost = array(
				'guid' => $guid,
				'code' => '',
				'info' => '',
				'data' => $data
			); 
			
            $result->guid = $guid;
            $result->info = $this->translate($result->info);
            
            $responseResult = array(
                'guid' => $result->guid,
                'code' => $result->code,
                'info' => $result->info,
                'data' => $result->data
            ); 

            $logparam['subject'] = 'API PERSON DELETESKILL';
            $logparam['identity'] = $user->id ? $user->id : '';
            $logparam['user'] = $user->id ? $user->id : '';
            $logparam['description'] = 'Log proses api person delete skill';
            $logparam['request'] = json_encode($requestPost);
            $logparam['response'] = json_encode($responseResult);
            
            $this->writeLogAction($logparam);
            
            return $this->getREST()->getResponse($result);
        } catch (Exception $ex) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
            return $this->getREST()->getResponse($result);
        }
    }
}
