<?php

namespace Neuron\Person\Relation\Storage;

use Zend\Db\Sql\Expression;
use Zend\Db\Sql\Predicate;

class Mysql extends \Neuron\Db\Storage implements Skeleton {

	protected $_nafManager = null;

	public function load($id, $mode = \Neuron\Person\Relation\Storage::LOADBY_ID) {
        
        /** kalau modif fungsi ini, silakan modif juga versi oracle dan versi postgre nya yah*/
        
        try {
            $select = $this->select()
                ->from(array('a'=> $this->_tables['person_relation']))
                ->join(
                    array('b' => $this->_tables['person_relation_status']), 
                    $this->_('relation_status_id', 'a.') .' = '. $this->_('relation_status_id', 'b.'), 
                    array('relation_status_name' => $this->_('name')), 
					'left'
                );

			$select->where($this->__(array('relation_id' => $id)));
            
            return $this->fetchRow($select);
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

	public function loadList($ids, $mode = \Neuron\Person\Relation\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
                ->from(array('a' => $this->_tables['person_relation']))
                ->join(
                    array('b' => $this->_tables['person_relation_status']), 
                    $this->_('relation_status_id', 'a.') .' = '. $this->_('relation_status_id', 'b.'), 
                    array('relation_status_name' => $this->_('name')), 
					'left'
                );

		$select->join(
					array('d' => $this->_tables['person']), 
					$this->_('person_id', 'a.') .' = '. $this->_('id', 'd.'), 
					array('*'), 
					'left'
				);

        $select->join(
            array('e' => $this->_tables['person_detail']), 
            $this->_('person_id', 'e.') .' = '. $this->_('id', 'd.'), 
            array('*'), 
            'left'
        );



        if(isset($params['start_date']) && !empty($params['start_date'])){
            $select->where->between(new Predicate\Expression('DATE('.$params['start_date'].')'), new Predicate\Expression('DATE(start_date)'), new Predicate\Expression('DATE(end_date)'));
        }

        if(isset($params['end_date']) && !empty($params['end_date'])){
            $select->where
            ->nest()
                ->lessThanOrEqualTo(new Predicate\Expression($this->_('end_date')), new Predicate\Expression('DATE('.$params['end_date'].')'))
                ->or
                ->isNull($this->_('end_date'))
            ->unnest();
        }else{
            $select->where->isNull($this->_('end_date'));
        }

        /* loadby page */
        if ($mode == \Neuron\Person\Relation\Storage::LISTBY_PAGE) {
			if(isset($params['person_id']) && !empty($params['person_id'])){
				$select->where(array($this->_('person_id', 'a.') => $params['person_id']));
			}

			if(isset($params['subject']) && !empty($params['subject'])){
				$select->where(array($this->_('subject', 'a.') => $params['subject']));
			}

            /* order */
            if(array_key_exists('order', $params) && array_key_exists('columns', $params)){
                foreach($params['order'] as $order){
                    $columnName = $params['columns'][$order['column']]['data'];
                    if($columnName == 'user_status_name'){
                        $colIdentity = $this->_('name', 'b.');
                    } else {
                        $colIdentity = $this->_($columnName, 'a.');
                    }
                    $select->order($colIdentity . ' ' . $order['dir']);
                }
            }
            
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('first_name', 'd.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('last_name', 'd.'), '%' . trim($ids) . '%')
                ->unnest();
            }

        } else {

            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                case \Neuron\Person\Relation\Storage::LISTBY_PERSON:
                    $select->where(array($this->_('person_id', 'a.') => $ids));
                    break;
                case \Neuron\Person\Relation\Storage::LISTBY_SUBJECT:
                    $select->where(array($this->_('subject', 'a.') => $ids));
                    break;
                default:
            }

        }

        $list = is_array($ids) ? $ids : ['id' => $ids];
        $list = is_array($params) ? array_merge($list, $params) : $list;
        $this->triggerQueryPerson($select, $list, __FUNCTION__);

        // echo str_replace('"','',$select->getSqlString()); die();
        try {
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

	public function save(\Neuron\Person\Relation $relation, $update = false) {
        try {

            /** issue sqlserver, primary key tidak usah dikirim untuk yang autoincrement */
            $data = $relation->pull();
            /** */
            
            /* mode update? */
            if ($update == true) {
                $update = $this->update()
                        ->table($this->_tables['person_relation'])
                        ->set($this->__($data))
                        ->where($this->__(array('relation_id' => $relation->id)));

                $this->triggerQueryPerson($update, $data, __FUNCTION__);
                
                if ($this->execute($update)) {

                    /* update sukses, return user id */
                    return $relation->id;
                }
            } else {

                $insert = $this->insert()
                        ->into($this->_tables['person_relation'])
                        ->autoincrement($this->_('relation_id'))
                        ->values($this->__($data));

                $this->triggerQueryPerson($insert, $data, __FUNCTION__);

                // echo str_replace('"','',$insert->getSqlString()); die();

                if ($id = $this->execute($insert)) {

                    /* insert sukses, return user id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

	public function terminate($id)
	{
		$update = $this->update()
				->table($this->_tables['person_relation'])
				->set(array('end_date'=> $this->now()))
				->where($this->__(array('relation_id' => $id)));

		if ($this->execute($update)) {

			/* update sukses, return user id */
			return $id;
		}
		return false;
	}

	private function triggerQueryPerson($query, $params, $caller) {

        /* trigger before query */
        if (($this->_nafManager === null) && defined('NAF_CAPABILITY_ADDON')) {
            if ($this->_nafManager = \Neuron\Application\Framework\Event\Manager::get()) {
                $this->_nafManager->events()->trigger('person_relation_query', $query, $params, $caller);
            }
        } else if(defined('NAF_CAPABILITY_ADDON')){
            $this->_nafManager->events()->trigger('person_relation_query', $query, $params, $caller);
        }
    }
}