<?php
namespace Neuron\Person;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Contact\Address\City;

class Skill extends \Neuron\Generic\Entity\Mapper {
    
    protected $_storage;

    public function __construct(\Neuron\Person\Skill\Storage\Skeleton $storage, $data = array()) {

        $this->_storage = $storage;

        parent::__construct(
            $data, 
            array(
                new Map('person_skill_id'),
                new Map('skill_code'),
                new Map('person_id'),
                new Map('skill_value'),
                new Map('start_date'),
                new Map('end_date'),
                new Map('document_id'),
                new Map('update_dtm'),
                new Map('update_by')
            ), 
            parent::CASE_INSENSITIVE
        );
    }
	
    public function load($id = null, $mode = Skill\Storage::LOADBY_ID, $params = array()) {
		if ($data = $this->_storage->load($id, $mode, $params)) {
			$this->push($data);
			return true;
		}

		return false;
    }

    public function save($update = false){
		if ($id = $this->_storage->save($this, $update)) {
			$this->person_skill_id = $id;
			return true;
		}
		
		return false;
    }

    public function delete($id) {
		/* init */
		$result = new Result();
		try {
			/* loaded? */
		
			if ($this->_storage->remove($id)) {

				$result->info = 'Success terminate skill';
			} else {

				//gagal
				$result->code = 51;
				$result->info = 'Failed terminate skill';

			}
				
			return $result;
		} catch (\Exception $ex) {
			error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
			$result->code = 500;
			$result->info = $ex->getMessage();
			return $result;
		}
    }

	public function create($data = array())
    {
		$result = new Result();

		$update = false;
		if ($data['person_skill_id']) {
			$update = true;
		}

		//create data relation
		$param = $this->pull();
		foreach ($data as $k => $v) {
			if (array_key_exists($k, $param)) {
				$param[$k] = $v;
			}
		}

		$this->push($param);
		if ($this->save($update)) {
			$result->data = $this->person_skill_id;
			$result->info = 'Success save skill';
		} else {
			//gagal
			$result->code = 51;
			$result->info = 'Failed save skill';
		}

		return $result;
    }
    
}